package cn.iocoder.yudao.module.wealth.service.receiptInvoice;

import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.i18n.core.LangEnum;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.api.internalMessage.InternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.wealth.convert.receiptInvoice.ReceiptInvoiceConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptInvoice.ReceiptInvoiceDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptInvoiceLog.ReceiptInvoiceLogDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptInvoice.ReceiptInvoiceMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptInvoiceLog.ReceiptInvoiceLogMapper;
import cn.iocoder.yudao.module.wealth.enums.InvoicingLinkEnum;
import cn.iocoder.yudao.module.wealth.enums.InvoicingStatusEnum;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.util.WealthGenCodeUtils;
import cn.iocoder.yudao.module.wealth.vo.receipt.ReceiptRelationOrderInfoVO;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.*;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBackVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.WealthMoneyAmountVO;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_NOT_EXISTS;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_VERIFICATION;
import static cn.iocoder.yudao.module.wealth.enums.InvoiceDataStatusEnum.REJECT;
import static cn.iocoder.yudao.module.wealth.enums.InvoiceDataStatusEnum.SUBMIT;
import static cn.iocoder.yudao.module.wealth.enums.InvoicingLinkEnum.REJECT_INVOICE_DATA;
import static cn.iocoder.yudao.module.wealth.enums.InvoicingStatusEnum.CANCELLED;
import static cn.iocoder.yudao.module.wealth.enums.InvoicingStatusEnum.NO_INVOICE;
import static cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum.*;

/**
 * 收款单开票信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ReceiptInvoiceServiceImpl extends AbstractService<ReceiptInvoiceMapper, ReceiptInvoiceDO> implements ReceiptInvoiceService {

    @Resource
    private ReceiptInvoiceMapper receiptInvoiceMapper;
    @Resource
    private ReceiptInvoiceLogMapper receiptInvoiceLogMapper;
    @Resource
    private ReceiptMapper receiptMapper;
    @Resource
    private ReceivableService receivableService;
    @Resource
    private WealthGenCodeUtils wealthGenCodeUtils;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private InternalMessageApi internalMessageApi;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private ReceiptService receiptService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createReceiptInvoice(InvoiceCreateReqVO createReqVO) {
        // 查询收款单
        ReceiptDO receiptDO = receiptMapper.selectById(createReqVO.getReceiptId());
        if (receiptDO == null) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        // 校验收款单状态
        checkReceiptForCreateInvoice(receiptDO);
        // 检查之前是否有取消的开票信息
        ReceiptInvoiceDO canceledInvoice = receiptInvoiceMapper.selectOne(Wrappers.lambdaQuery(ReceiptInvoiceDO.class)
                .eq(ReceiptInvoiceDO::getReceiptId, receiptDO.getId())
                .isNotNull(ReceiptInvoiceDO::getCancelUserId));
        if (canceledInvoice != null) {
            receiptInvoiceMapper.deleteById(canceledInvoice.getId());
        }
        // 创建开票资料
        ReceiptInvoiceDO receiptInvoiceDO = ReceiptInvoiceConvert.INSTANCE.convert(createReqVO);
        receiptInvoiceDO.setInvoiceNo(wealthGenCodeUtils.generateInvoiceCode());
        receiptInvoiceDO.setReceiptId(receiptDO.getId());
        receiptInvoiceDO.setReceiptNo(receiptDO.getReceiptNo());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        receiptInvoiceDO.setApplyUserId(loginUser.getId());
        receiptInvoiceDO.setApplyUserName(loginUser.getUsername());
        receiptInvoiceMapper.insert(receiptInvoiceDO);
        // 更新收款单状态
        receiptDO.setInvoiceId(receiptInvoiceDO.getId());
        receiptDO.setOpenInvoice(1);
        receiptDO.setInvoicingStatus(InvoicingStatusEnum.WAIT.getValue());
        receiptDO.setInvoiceDataStatus(SUBMIT.getValue());
        receiptMapper.updateById(receiptDO);
        // 记录日志
        recordLog(receiptInvoiceDO.getId(), InvoicingLinkEnum.APPLY_INVOICE, createReqVO.getInvoiceRemark(), null);
        return receiptInvoiceDO.getId();
    }

    private void checkReceiptForCreateInvoice(ReceiptDO receiptDO) {
        //【申请开票】按钮：新增申请开票按钮，当收款单状态≠草稿且≠待审批且≠审批拒绝，开票状态=已取消或者无申请记录，按钮显示且可点击，跳转申请开票页面
        if (Objects.equals(receiptDO.getState(), DRAFT.getValue()) || Objects.equals(receiptDO.getState(), APPROVE_ING.getValue()) || Objects.equals(receiptDO.getState(), APPROVE_FAIL.getValue())) {
            throw exception(RECEIPT_VERIFICATION);
        }
        if (!Objects.equals(receiptDO.getInvoicingStatus(), CANCELLED.getValue()) && !Objects.equals(receiptDO.getInvoicingStatus(), NO_INVOICE.getValue())) {
            throw exception(RECEIPT_VERIFICATION);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceiptInvoiceData(ReceiptInvoiceUpdateDataReqVO updateReqVO) {
        // 校验存在
        this.validateReceiptInvoiceExists(updateReqVO.getId());
        // TODO 校验收款单开票状态为待开票
        // 更新开票资料
        ReceiptInvoiceDO updateObj = ReceiptInvoiceConvert.INSTANCE.convert(updateReqVO);
        receiptInvoiceMapper.updateById(updateObj);
        // 记录日志
        recordLog(updateReqVO.getId(), InvoicingLinkEnum.MODIFY_INVOICE_DATA, updateReqVO.getInvoiceRemark(), null);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceiptInvoiceInfo(ReceiptInvoiceUpdateInfoReqVO updateReqVO) {
        ReceiptInvoiceDO receiptInvoiceDO = receiptInvoiceMapper.selectById(updateReqVO.getId());
        if (receiptInvoiceDO == null) {
            throw exception(new ErrorCode(400, "收款单开票信息不存在"));
        }
        ReceiptInvoiceDO updateObj = ReceiptInvoiceConvert.INSTANCE.convert(updateReqVO);
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        updateObj.setInvoicerId(loginUser.getId());
        updateObj.setInvoiceInfoSaveTime(new Date());
        receiptInvoiceMapper.updateById(updateObj);
        ReceiptDO receiptDO = new ReceiptDO();
        receiptDO.setId(receiptInvoiceDO.getReceiptId());
        receiptDO.setInvoicingStatus(InvoicingStatusEnum.INVOICED.getValue());
        receiptMapper.updateById(receiptDO);
        // 记录日志
        InvoicingLinkEnum invoicingLinkEnum = receiptInvoiceDO.getInvoiceNumber() == null ? InvoicingLinkEnum.SUBMIT_INVOICE : InvoicingLinkEnum.MODIFY_INVOICE_INFO;
        recordLog(updateReqVO.getId(), invoicingLinkEnum, updateReqVO.getInvoiceInfoRemark(), null);
    }

    private void validateReceiptInvoiceExists(Long id) {
        if (receiptInvoiceMapper.selectById(id) == null) {
            throw exception(new ErrorCode(400, "收款单开票信息不存在"));
        }
    }

    @Override
    public ReceiptInvoiceDetailVO getReceiptInvoiceDetail(Long id) {
        ReceiptInvoiceDetailVO receiptInvoiceDetail = receiptInvoiceMapper.getReceiptInvoiceDetail(id);
        if (receiptInvoiceDetail == null) {
            return null;
        }
        List<ReceivableBackVO> receivableList = receivableService.getReceivableListByReceiptId(receiptInvoiceDetail.getReceiptId());
        receiptInvoiceDetail.setReceivableList(receivableList);
        // 根据应收明细统计收款单信息
        receiptInvoiceDetail.setBaseCurrencyId(receivableList.get(0).getBaseCurrencyId());
        statisticsReceiptInfo(receiptInvoiceDetail, receivableList);
        // 查询日志
        List<ReceiptInvoiceLogDO> invoiceLogDOS = receiptInvoiceLogMapper.selectList("invoice_id", id);
        receiptInvoiceDetail.setInvoiceLog(ReceiptInvoiceConvert.INSTANCE.logDOtoLogVOList(invoiceLogDOS));
        List<ReceiptRelationOrderInfoVO> orderInfoList = receiptMapper.getOrderInfoByReceiptId(receiptInvoiceDetail.getReceiptId());
        receiptInvoiceDetail.setOrderInfoList(orderInfoList);

        receiptInvoiceDetail.setCreator(adminUserApi.getUser(Long.parseLong(receiptInvoiceDetail.getCreator())).getNickname());
        receiptInvoiceDetail.setUpdater(adminUserApi.getUser(Long.parseLong(receiptInvoiceDetail.getUpdater())).getNickname());
        // 设置收款人,核销人
        receiptInvoiceDetail.setPayee(receiptInvoiceDetail.getCreator());
        String approver = receiptService.getReceiptApprover(receiptInvoiceDetail.getReceiptId());
        receiptInvoiceDetail.setWriteOffUser(approver);
        return receiptInvoiceDetail;
    }

    @Override
    public ReceiptInvoiceDetailVO getReceiptDetailForCreateInvoice(Long receiptId) {
        ReceiptDO receiptDO = receiptMapper.selectById(receiptId);
        ReceiptInvoiceDetailVO receiptInvoiceDetail = ReceiptInvoiceConvert.INSTANCE.convert(receiptDO);
        receiptInvoiceDetail.setReceiptId(receiptDO.getId());
        receiptInvoiceDetail.setUpdater(adminUserApi.getUser(Long.parseLong(receiptInvoiceDetail.getUpdater())).getNickname());
        List<ReceivableBackVO> receivableList = receivableService.getReceivableListByReceiptId(receiptInvoiceDetail.getReceiptId());
        receiptInvoiceDetail.setReceivableList(receivableList);
        // 根据应收明细统计收款单信息
        receiptInvoiceDetail.setBaseCurrencyId(receivableList.get(0).getBaseCurrencyId());
        statisticsReceiptInfo(receiptInvoiceDetail, receivableList);
        receiptInvoiceDetail.setPayee(receiptInvoiceDetail.getCreator());
        String approver = receiptService.getReceiptApprover(receiptInvoiceDetail.getReceiptId());
        receiptInvoiceDetail.setWriteOffUser(approver);
        return receiptInvoiceDetail;
    }

    private void statisticsReceiptInfo(ReceiptInvoiceDetailVO receiptInvoiceDetail, List<ReceivableBackVO> receivableList) {
        // 应收明细按币种分组，统计每个币种的应收总金额和核销总金额
        Map<Long, List<ReceivableBackVO>> receiptAmountMap = receivableList.stream().collect(Collectors.groupingBy(ReceivableBackVO::getCurrencyId));
        List<WealthMoneyAmountVO> receiptAmountList = new ArrayList<>();
        Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();
        for (Map.Entry<Long, List<ReceivableBackVO>> entry : receiptAmountMap.entrySet()) {
            Long currencyId = entry.getKey();
            List<ReceivableBackVO> currencyReceivableList = entry.getValue();
            WealthMoneyAmountVO wealthMoneyAmountVO = new WealthMoneyAmountVO();
            wealthMoneyAmountVO.setCurrencyId(currencyId.intValue());
            wealthMoneyAmountVO.setCurrencySymbol(currencyMap.get(currencyId.intValue()).getFuhao());
            wealthMoneyAmountVO.setCurrencyName(currencyMap.get(currencyId.intValue()).getTitleZh());
            wealthMoneyAmountVO.setTotalAmount(currencyReceivableList.stream().map(ReceivableBackVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add));
            wealthMoneyAmountVO.setNotWriteOffAmount(currencyReceivableList.stream().map(ReceivableBackVO::getNotWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add));
            wealthMoneyAmountVO.setWriteOffAmount(currencyReceivableList.stream().map(ReceivableBackVO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add));
            wealthMoneyAmountVO.setReceivableTotalAmount(currencyReceivableList.stream().map(vo -> vo.getTaxAmount().subtract(vo.getDiscountTotal())).reduce(BigDecimal.ZERO, BigDecimal::add));
            receiptAmountList.add(wealthMoneyAmountVO);
        }
        receiptInvoiceDetail.setReceiptAmount(receiptAmountList);
        receiptInvoiceDetail.setReceivableTotalAmountUsd(BigDecimal.ZERO);
        receiptInvoiceDetail.setWriteOffAmountUsd(BigDecimal.ZERO);
        for (WealthMoneyAmountVO wealthMoneyAmountVO : receiptAmountList) {
            ExchangeRateRespDTO rate = currencyApi.getCurrencyRateByCode(wealthMoneyAmountVO.getCurrencySymbol(), currencyMap.get(receiptInvoiceDetail.getBaseCurrencyId().intValue()).getFuhao());
            receiptInvoiceDetail.setReceivableTotalAmountUsd(receiptInvoiceDetail.getReceivableTotalAmountUsd().add(wealthMoneyAmountVO.getReceivableTotalAmount().multiply(rate.getCurrencyRate())));
            receiptInvoiceDetail.setWriteOffAmountUsd(receiptInvoiceDetail.getWriteOffAmountUsd().add(wealthMoneyAmountVO.getWriteOffAmount().multiply(rate.getCurrencyRate())));
        }
        BigDecimal writeOffProportion = receiptInvoiceDetail.getWriteOffAmountUsd().divide(receiptInvoiceDetail.getReceivableTotalAmountUsd(), 2, RoundingMode.HALF_UP).multiply(new BigDecimal("100"));
        receiptInvoiceDetail.setWriteOffProportion(writeOffProportion + "%");
        if (receiptInvoiceDetail.getCurrencyId() != null) {
            receiptInvoiceDetail.setCurrencySymbol(currencyMap.get(receiptInvoiceDetail.getCurrencyId().intValue()).getFuhao());
        }
        // 应收只有一种币种且为人民币时，默认赋值为应收总金额
        if (receiptAmountList.size() == 1 && "￥".equals(receiptAmountList.get(0).getCurrencySymbol())) {
            receiptInvoiceDetail.setInvoiceMoney(receiptAmountList.get(0).getReceivableTotalAmount());
        }
    }

    @Override
    public List<ReceiptInvoiceDO> getReceiptInvoiceList(Collection<Long> ids) {
        return receiptInvoiceMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceiptInvoicePageBackVO> getReceiptInvoicePage(ReceiptInvoiceQueryVO query, PageVO page) {
        IPage<ReceiptInvoicePageBackVO> mpPage = MyBatisUtils.buildPage(page);
        receiptInvoiceMapper.getReceiptInvoicePage(mpPage, query);
        mpPage.getRecords().forEach(receiptInvoicePageBackVO -> {
            String writeOffProportion = receiptInvoicePageBackVO.getWriteOffProportion();
            if (StringUtils.isBlank(writeOffProportion)) {
                receiptInvoicePageBackVO.setWriteOffProportion("0.00%");
            } else {
                receiptInvoicePageBackVO.setWriteOffProportion(new BigDecimal(writeOffProportion).setScale(2, RoundingMode.HALF_UP) + "%");
            }
            List<ReceiptRelationOrderInfoVO> orderInfoByReceiptId = receiptMapper.getOrderInfoByReceiptId(receiptInvoicePageBackVO.getReceiptId());
            receiptInvoicePageBackVO.setOrderInfoList(orderInfoByReceiptId);
        });
        return PageResult.of(mpPage);
    }

    @Override
    public List<ReceiptInvoiceDO> getReceiptInvoiceList(ReceiptInvoiceQueryVO query) {
        return receiptInvoiceMapper.selectList(query);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean rejectInvoiceInfo(ReceiptInvoiceOperateReqVO reqVO) {
        ReceiptInvoiceDO receiptInvoiceDO = receiptInvoiceMapper.selectById(reqVO.getId());
        if (receiptInvoiceDO == null) {
            throw new ServiceException(new ErrorCode(400, "发票不存在"));
        }
        ReceiptDO receiptDO = new ReceiptDO();
        receiptDO.setId(receiptInvoiceDO.getReceiptId());
        receiptDO.setInvoiceDataStatus(REJECT.getValue());
        receiptMapper.updateById(receiptDO);
        recordLog(receiptInvoiceDO.getId(), REJECT_INVOICE_DATA, reqVO.getRemark(), null);
        sendMessage(reqVO.getNotifyUserIdList(), REJECT_INVOICE_DATA, receiptInvoiceDO.getReceiptNo(), receiptInvoiceDO.getInvoiceHeader());
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean cancelInvoice(ReceiptInvoiceOperateReqVO reqVO) {
        ReceiptInvoiceDO receiptInvoiceDO = receiptInvoiceMapper.selectById(reqVO.getId());
        if (receiptInvoiceDO == null) {
            throw new ServiceException(new ErrorCode(400, "发票不存在"));
        }
        ReceiptDO receiptDO = new ReceiptDO();
        receiptDO.setId(receiptInvoiceDO.getReceiptId());
        receiptDO.setOpenInvoice(0);
        receiptDO.setInvoicingStatus(CANCELLED.getValue());
        receiptDO.setInvoiceDataStatus(REJECT.getValue());
        receiptMapper.updateById(receiptDO);
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        receiptInvoiceDO.setCancelUserId(loginUser.getId());
        receiptInvoiceDO.setCancelTime(new Date());
        receiptInvoiceMapper.updateById(receiptInvoiceDO);
        recordLog(receiptInvoiceDO.getId(), InvoicingLinkEnum.CANCEL_INVOICE, reqVO.getRemark(), null);
        sendMessage(reqVO.getNotifyUserIdList(), InvoicingLinkEnum.CANCEL_INVOICE, receiptInvoiceDO.getReceiptNo(), receiptInvoiceDO.getInvoiceHeader());
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteInvoiceInfo(ReceiptInvoiceOperateReqVO reqVO) {
        ReceiptInvoiceDO receiptInvoiceDO = receiptInvoiceMapper.selectById(reqVO.getId());
        if (receiptInvoiceDO == null) {
            throw new ServiceException(new ErrorCode(400, "发票不存在"));
        }
        // 清空发票号码，开票类型，开票金额，开票人，开票时间
        LambdaUpdateWrapper<ReceiptInvoiceDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(ReceiptInvoiceDO::getId, receiptInvoiceDO.getId())
                .set(ReceiptInvoiceDO::getInvoiceNumber, null)
                .set(ReceiptInvoiceDO::getInvoiceTypeId, null)
                .set(ReceiptInvoiceDO::getInvoiceMoney, null)
                .set(ReceiptInvoiceDO::getCurrencyId, null)
                .set(ReceiptInvoiceDO::getInvoicerId, null)
                .set(ReceiptInvoiceDO::getInvoiceTime, null)
                .set(ReceiptInvoiceDO::getInvoiceInfoRemark, null)
                .set(ReceiptInvoiceDO::getInvoiceInfoSaveTime, new Date());
        receiptInvoiceMapper.update(receiptInvoiceDO, updateWrapper);

        ReceiptDO receiptDO = new ReceiptDO();
        receiptDO.setId(receiptInvoiceDO.getReceiptId());
        receiptDO.setInvoicingStatus(InvoicingStatusEnum.WAIT.getValue());
        receiptMapper.updateById(receiptDO);
        recordLog(receiptInvoiceDO.getId(), InvoicingLinkEnum.DELETE_INVOICE_INFO, reqVO.getRemark(), null);
        return true;
    }

    /**
     * 记录日志
     *
     * @param invoiceId 发票id
     * @param linkEnum  环节
     * @param remark    备注
     * @param bpmStatus 审批流程
     */
    private void recordLog(Long invoiceId, InvoicingLinkEnum linkEnum, String remark, Integer bpmStatus) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String username = "";
        if (user != null && user.getUsername() != null) {
            username = user.getUsername();
        }
        ReceiptInvoiceLogDO receiptInvoiceLogDO = ReceiptInvoiceLogDO.builder()
                .invoiceId(invoiceId)
                .operator(username)
                .remark(remark)
                .invoicingLink(linkEnum.getValue())
                .bpmStatus(bpmStatus)
                .createTime(new Date())
                .build();
        receiptInvoiceLogMapper.insert(receiptInvoiceLogDO);
    }

    /**
     * 发送站内信
     */
    private void sendMessage(List<Long> userIdList, InvoicingLinkEnum linkEnum, String receiptNo, String invoiceHeader) {
        if (userIdList == null || userIdList.isEmpty()) {
            return;
        }
        String title = "";
        String titleEn = "";
        String titleFr = "";
        String content = "";
        String contentEn = "";
        String contentFr = "";
        switch (linkEnum) {
            case REJECT_INVOICE_DATA:
                title = String.format(I18nMessage.getMessage("app.message.reject.invoice.title", LangEnum.ZH.getLang()), receiptNo, invoiceHeader);
                titleEn = String.format(I18nMessage.getMessage("app.message.reject.invoice.title", LangEnum.EN.getLang()), receiptNo, invoiceHeader);
                titleFr = String.format(I18nMessage.getMessage("app.message.reject.invoice.title", LangEnum.FR.getLang()), receiptNo, invoiceHeader);
                content = String.format(I18nMessage.getMessage("app.message.reject.invoice.content", LangEnum.ZH.getLang()), receiptNo, invoiceHeader);
                contentEn = String.format(I18nMessage.getMessage("app.message.reject.invoice.content", LangEnum.EN.getLang()), receiptNo, invoiceHeader);
                contentFr = String.format(I18nMessage.getMessage("app.message.reject.invoice.content", LangEnum.FR.getLang()), receiptNo, invoiceHeader);
                break;
            case CANCEL_INVOICE:
                title = String.format(I18nMessage.getMessage("app.message.cancel.invoice.title", LangEnum.ZH.getLang()), receiptNo, invoiceHeader);
                titleEn = String.format(I18nMessage.getMessage("app.message.cancel.invoice.title", LangEnum.EN.getLang()), receiptNo, invoiceHeader);
                titleFr = String.format(I18nMessage.getMessage("app.message.cancel.invoice.title", LangEnum.FR.getLang()), receiptNo, invoiceHeader);
                content = String.format(I18nMessage.getMessage("app.message.cancel.invoice.content", LangEnum.ZH.getLang()), receiptNo, invoiceHeader);
                contentEn = String.format(I18nMessage.getMessage("app.message.cancel.invoice.content", LangEnum.EN.getLang()), receiptNo, invoiceHeader);
                contentFr = String.format(I18nMessage.getMessage("app.message.cancel.invoice.content", LangEnum.FR.getLang()), receiptNo, invoiceHeader);
                break;
        }
        // 发送站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(userIdList)
                .fromId("")
                .sendTime(new Date())
                .title(title)
                .titleEn(titleEn)
                .titleFr(titleFr)
                .content(content)
                .contentEn(contentEn)
                .contentFr(contentFr)
                .type(linkEnum == REJECT_INVOICE_DATA ? 4 : 5).build();
        internalMessageApi.createInternalMessage(dto);
    }
}
